==> openbot-1.0.9-sources.jar: ai/engagely/openbot/model/utils/helpers/BotViewHelper.kt <==
package ai.engagely.openbot.model.utils.helpers

import ai.engagely.openbot.R
import ai.engagely.openbot.model.constants.ViewConstants
import ai.engagely.openbot.model.pojos.internal.botsettings.IBotChatWindowConfig
import ai.engagely.openbot.model.utils.exts.showToast
import ai.engagely.openbot.model.utils.general.LogUtils
import ai.engagely.openbot.model.utils.general.ViewUtils
import ai.engagely.openbot.view.activities.BotChatActivity
import ai.engagely.openbot.view.activities.ExoPlayerActivity
import ai.engagely.openbot.view.activities.PermissionCompatActivity
import ai.engagely.openbot.view.customviews.BotContainerView
import ai.engagely.openbot.view.customviews.WelcomeVideoView
import ai.engagely.openbot.viewmodel.OpenBotViewModel
import ai.engagely.openbot.viewmodel.VideoPlayerViewModel
import android.app.Activity
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.os.Build
import android.view.View
import androidx.activity.result.contract.ActivityResultContracts
import androidx.fragment.app.FragmentActivity
import androidx.lifecycle.ViewModelProvider
import androidx.localbroadcastmanager.content.LocalBroadcastManager

class BotViewHelper(
    activity: FragmentActivity,
    private val botViewContainerView: BotContainerView?
) : PermissionCompatActivity.ReadExternalStoragePermissionListener {

    private val openBotViewModel by lazy {
        ViewModelProvider(activity)[OpenBotViewModel::class.java]
    }

    private val playerViewModel by lazy {
        ViewModelProvider(activity)[VideoPlayerViewModel::class.java]
    }

    private val chatResult =
        activity.registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
            it?.let { activityResult ->
                openBotViewModel.resetWelcomeData()
                activityResult.data?.getParcelableExtra<IBotChatWindowConfig>(IBotChatWindowConfig.UNIQUE_NAME)
                    ?.let { botConfigData ->
                        openBotViewModel.updateBotConfigData(botConfigData)
                    }
                when (activityResult.resultCode) {
                    RESULT_SESSION_UPDATE_REQUESTED -> {
                        openBotViewModel.refreshSession()
                    }
                }
            }
        }

    private val exoPlayerResult =
        activity.registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { activityResult ->
            if (Activity.RESULT_OK == activityResult.resultCode) {
                activityResult.data?.let { resultData ->
                    resultData.getStringExtra(ExoPlayerActivity.VIDEO_URL_TAG)?.let { url ->
                        playerViewModel.updateInitialData(
                            resultData.getLongExtra(
                                ExoPlayerActivity.VIDEO_CURRENT_POSITION_TAG,
                                0
                            ),
                            resultData.getBooleanExtra(
                                ExoPlayerActivity.VIDEO_PLAY_WHEN_READY_TAG,
                                false
                            ),
                            resultData.getFloatExtra(
                                ExoPlayerActivity.VIDEO_PLAYBACK_SPEED_TAG,
                                1f
                            ),
                            url
                        )
                        scrollWelcomeDataToEnd()
                    }
                }
            }
        }

    private val videoPlayerPIPBroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            intent?.let { resultData ->
                val isInPictureInPictureMode: Boolean = resultData.getBooleanExtra(
                    ExoPlayerActivity.VIDEO_IS_IN_PICTURE_IN_PICTURE_MODE,
                    false
                )
                playerViewModel.setPictureInPictureMode(isInPictureInPictureMode)
            }
        }
    }

    private val videoPlayerPositionBroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            intent?.let { resultData ->
                playerViewModel.updateCurrentData(
                    resultData.getLongExtra(
                        ExoPlayerActivity.VIDEO_CURRENT_POSITION_TAG,
                        0
                    ),
                    resultData.getFloatExtra(
                        ExoPlayerActivity.VIDEO_PLAYBACK_SPEED_TAG,
                        1f
                    )
                )
            }
        }
    }

    init {
        activity.lifecycle.addObserver(playerViewModel)
        setObservers(activity)
        setClickListeners(activity)
        setBroadcastReceivers(activity)
    }

    private fun setBroadcastReceivers(activity: FragmentActivity) {
        LocalBroadcastManager.getInstance(activity).registerReceiver(
            videoPlayerPIPBroadcastReceiver,
            IntentFilter(ExoPlayerActivity.VIDEO_PLAYER_PIP_BROADCAST_EVENT)
        )
        LocalBroadcastManager.getInstance(activity).registerReceiver(
            videoPlayerPositionBroadcastReceiver,
            IntentFilter(ExoPlayerActivity.VIDEO_PLAYER_POSITION_BROADCAST_EVENT)
        )
    }

    fun init(botId: String, extras: Map<String, String>? = null) {
        openBotViewModel.init(botId, extras)
        openBotViewModel.fetchData()
    }

    private fun setClickListeners(activity: FragmentActivity) {
        botViewContainerView?.getBotView()?.setImageClickListener {
            val botChatConfig = openBotViewModel.botChatWindowConfig
            botChatConfig?.let {
                chatResult.launch(Intent(activity, BotChatActivity::class.java).apply {
                    putExtra(IBotChatWindowConfig.UNIQUE_NAME, botChatConfig)
                    putExtras(openBotViewModel.getExtras())
                })
            }
        }
        botViewContainerView?.getBotView()?.setImageLongClickListener {
            if (openBotViewModel.welcomeDataLiveData.value?.isNotEmpty() == true) {
                botViewContainerView.getWelcomeData()?.visibility = View.VISIBLE
                scrollWelcomeDataToEnd()
                true
            } else {
                false
            }
        }
        botViewContainerView?.getBotView()?.setErrorRetryClickListener {
            openBotViewModel.fetchData()
        }
    }

    private fun setObservers(activity: FragmentActivity) {
        openBotViewModel.isLoadingInitialApi.observe(activity) { value ->
            botViewContainerView?.getBotView()?.setLoading(value)
        }
        openBotViewModel.imageBitmapLiveData.observe(activity) { value ->
            value?.let { bitmap ->
                botViewContainerView?.getBotView()?.setImage(bitmap)
            }
        }
        openBotViewModel.errorStateLiveData.observe(activity) {
            botViewContainerView?.getBotView()?.setErrorView(it != null)
        }
        openBotViewModel.welcomeDataLiveData.observe(activity) {
            if (it?.isNotEmpty() == true) {
                val themeColor = ViewUtils.parseColorSafely(
                    openBotViewModel.botChatWindowConfig?.botThemColor,
                    ViewConstants.VIEW_DEFAULT_COLOR_STRING_DARK
                )
                botViewContainerView?.getWelcomeData()
                    ?.setData(it, themeColor, object : WelcomeVideoView.OnPlayClickListener {
                        override fun onPlayClicked(url: String?) {
                            playerViewModel.startPlaying(url)
                        }

                        override fun onFullScreenClicked() {
                            openFullScreenPlayer(activity, false)
                        }

                        override fun onVolumeToggleClicked() {
                            playerViewModel.toggleVolume()
                        }

                        override fun onPlaybackSpeedChanged(speed: Float) {
                            playerViewModel.setPlaybackSpeed(speed)
                        }

                        override fun onDownloadMenuSelected() {
                            (activity as? PermissionCompatActivity)?.requestReadExternalStoragePermission(
                                REQUEST_READ_EXTERNAL_STORAGE_PERMISSION,
                                this@BotViewHelper
                            )
                        }

                        override fun onPictureInPictureSelected() {
                            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N
                                && activity.packageManager.hasSystemFeature(PackageManager.FEATURE_PICTURE_IN_PICTURE)
                            ) {
                                openFullScreenPlayer(activity, true)
                            } else {
                                activity.showToast(activity.getString(R.string.service_unavailable))
                            }
                        }
                    })
                botViewContainerView?.getWelcomeData()?.visibility = View.VISIBLE
                botViewContainerView?.getWelcomeData()?.setPlayer(playerViewModel.player)
                botViewContainerView?.getWelcomeData()
                    ?.setVolumeMuted(playerViewModel.volumeMutedLiveData.value)
                botViewContainerView?.getWelcomeData()
                    ?.setPlaybackSpeed(playerViewModel.playbackSpeedLiveData.value ?: 1f)
                botViewContainerView?.getWelcomeData()
                    ?.setPlayerPIPMode(playerViewModel.pictureInPictureModeLiveData.value)
                scrollWelcomeDataToEnd()
            } else {
                botViewContainerView?.getWelcomeData()?.visibility = View.GONE
            }
        }
        openBotViewModel.welcomeVideoUrlLiveData.observe(activity) { url ->
            playerViewModel.startPlaying(url)
        }
        playerViewModel.playerLiveData.observe(activity) {
            botViewContainerView?.getWelcomeData()?.setPlayer(it)
            scrollWelcomeDataToEnd()
        }
        playerViewModel.volumeMutedLiveData.observe(activity) {
            botViewContainerView?.getWelcomeData()?.setVolumeMuted(it)
        }
        playerViewModel.playbackSpeedLiveData.observe(activity) {
            botViewContainerView?.getWelcomeData()
                ?.setPlaybackSpeed(it ?: 1f)
        }
        playerViewModel.pictureInPictureModeLiveData.observe(activity) {
            botViewContainerView?.getWelcomeData()?.setPlayerPIPMode(it)
        }
        openBotViewModel.botImageDimensionsLiveData.observe(activity) { botDimension ->
            botViewContainerView?.getBotView()?.setBotDimensions(botDimension)
        }
    }

    private fun openFullScreenPlayer(activity: FragmentActivity, playInPIP: Boolean) {
        playerViewModel.getUrl()?.let { url ->
            val playerSize = botViewContainerView?.getWelcomeData()?.getPlayerSize()
            ExoPlayerActivity.getRequestIntent(
                activity,
                playerViewModel.getCurrentPosition(),
                playerViewModel.getPlayWhenReady(),
                playerViewModel.getPlaybackSpeed(),
                url,
                playInPIP,
                playerSize?.width ?: 0,
                playerSize?.height ?: 0,
                playerViewModel.player?.isPlaying,
                true
            ).also { intent ->
                playerViewModel.stopPlaying()
                exoPlayerResult.launch(intent)
            }
        }
    }

    private fun scrollWelcomeDataToEnd() {
        botViewContainerView?.getWelcomeDataContainer()?.postDelayed({
            botViewContainerView.getWelcomeDataContainer()?.fullScroll(View.FOCUS_DOWN)
        }, 600)
    }

    fun onConfigurationChanged(newConfig: Configuration) {
        scrollWelcomeDataToEnd()
    }

    override fun onPermissionResponse(
        requestCode: Int,
        readExternalStoragePermission: PermissionCompatActivity.ReadExternalStoragePermission
    ) {
        if (readExternalStoragePermission == PermissionCompatActivity.ReadExternalStoragePermission.GRANTED) {
            if (requestCode == REQUEST_READ_EXTERNAL_STORAGE_PERMISSION) {
                playerViewModel.downloadVideo()
            }
        }
    }

    fun onDestroy(activity: FragmentActivity) {
        try {
            LocalBroadcastManager.getInstance(activity)
                .unregisterReceiver(videoPlayerPIPBroadcastReceiver)
            LocalBroadcastManager.getInstance(activity)
                .unregisterReceiver(videoPlayerPositionBroadcastReceiver)
        } catch (e: Exception) {
            LogUtils.logException(e)
        }
    }

    companion object {
        const val RESULT_SESSION_UPDATE_REQUESTED = 999
        const val RESULT_BOT_CONFIG_UPDATED = 1000
        const val REQUEST_READ_EXTERNAL_STORAGE_PERMISSION = 1001
    }
}

==> openbot-1.0.11-sources.jar: ai/engagely/openbot/model/utils/helpers/BotViewHelper.kt <==
package ai.engagely.openbot.model.utils.helpers

import ai.engagely.openbot.R
import ai.engagely.openbot.model.constants.ViewConstants
import ai.engagely.openbot.model.pojos.internal.botsettings.IBotChatWindowConfig
import ai.engagely.openbot.model.utils.exts.showToast
import ai.engagely.openbot.model.utils.general.LogUtils
import ai.engagely.openbot.model.utils.general.ViewUtils
import ai.engagely.openbot.view.activities.BotChatActivity
import ai.engagely.openbot.view.activities.ExoPlayerActivity
import ai.engagely.openbot.view.activities.PermissionCompatActivity
import ai.engagely.openbot.view.customviews.BotContainerView
import ai.engagely.openbot.view.customviews.WelcomeVideoView
import ai.engagely.openbot.viewmodel.OpenBotViewModel
import ai.engagely.openbot.viewmodel.VideoPlayerViewModel
import android.app.Activity
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.os.Build
import android.view.View
import androidx.activity.result.contract.ActivityResultContracts
import androidx.fragment.app.FragmentActivity
import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.ViewModelProvider
import androidx.localbroadcastmanager.content.LocalBroadcastManager

class BotViewHelper(
    activity: FragmentActivity,
    private val botViewContainerView: BotContainerView?
) : PermissionCompatActivity.ReadExternalStoragePermissionListener, DefaultLifecycleObserver {

    private val openBotViewModel by lazy {
        ViewModelProvider(activity)[OpenBotViewModel::class.java]
    }

    private val playerViewModel by lazy {
        ViewModelProvider(activity)[VideoPlayerViewModel::class.java]
    }

    private val chatResult =
        activity.registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
            it?.let { activityResult ->
                openBotViewModel.resetWelcomeData()
                activityResult.data?.getParcelableExtra<IBotChatWindowConfig>(IBotChatWindowConfig.UNIQUE_NAME)
                    ?.let { botConfigData ->
                        openBotViewModel.updateBotConfigData(botConfigData)
                    }
                when (activityResult.resultCode) {
                    RESULT_SESSION_UPDATE_REQUESTED -> {
                        openBotViewModel.refreshSession()
                    }
                }
            }
        }

    private val exoPlayerResult =
        activity.registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { activityResult ->
            if (Activity.RESULT_OK == activityResult.resultCode) {
                activityResult.data?.let { resultData ->
                    resultData.getStringExtra(ExoPlayerActivity.VIDEO_URL_TAG)?.let { url ->
                        playerViewModel.updateInitialData(
                            resultData.getLongExtra(
                                ExoPlayerActivity.VIDEO_CURRENT_POSITION_TAG,
                                0
                            ),
                            resultData.getBooleanExtra(
                                ExoPlayerActivity.VIDEO_PLAY_WHEN_READY_TAG,
                                false
                            ),
                            resultData.getFloatExtra(
                                ExoPlayerActivity.VIDEO_PLAYBACK_SPEED_TAG,
                                1f
                            ),
                            url
                        )
                        scrollWelcomeDataToEnd()
                    }
                }
            }
        }

    private val videoPlayerPIPBroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            intent?.let { resultData ->
                val isInPictureInPictureMode: Boolean = resultData.getBooleanExtra(
                    ExoPlayerActivity.VIDEO_IS_IN_PICTURE_IN_PICTURE_MODE,
                    false
                )
                playerViewModel.setPictureInPictureMode(isInPictureInPictureMode)
            }
        }
    }

    private val videoPlayerPositionBroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            intent?.let { resultData ->
                playerViewModel.updateCurrentData(
                    resultData.getLongExtra(
                        ExoPlayerActivity.VIDEO_CURRENT_POSITION_TAG,
                        0
                    ),
                    resultData.getFloatExtra(
                        ExoPlayerActivity.VIDEO_PLAYBACK_SPEED_TAG,
                        1f
                    )
                )
            }
        }
    }

    init {
        activity.lifecycle.addObserver(playerViewModel)
        activity.lifecycle.addObserver(this)
        setObservers(activity)
        setClickListeners(activity)
        setBroadcastReceivers(activity)
    }

    private fun setBroadcastReceivers(activity: FragmentActivity) {
        LocalBroadcastManager.getInstance(activity).registerReceiver(
            videoPlayerPIPBroadcastReceiver,
            IntentFilter(ExoPlayerActivity.VIDEO_PLAYER_PIP_BROADCAST_EVENT)
        )
        LocalBroadcastManager.getInstance(activity).registerReceiver(
            videoPlayerPositionBroadcastReceiver,
            IntentFilter(ExoPlayerActivity.VIDEO_PLAYER_POSITION_BROADCAST_EVENT)
        )
    }

    fun init(botId: String, extras: Map<String, String>? = null) {
        openBotViewModel.init(botId, extras)
        openBotViewModel.fetchData()
    }

    private fun setClickListeners(activity: FragmentActivity) {
        botViewContainerView?.getBotView()?.setImageClickListener {
            val botChatConfig = openBotViewModel.botChatWindowConfig
            botChatConfig?.let {
                chatResult.launch(Intent(activity, BotChatActivity::class.java).apply {
                    putExtra(IBotChatWindowConfig.UNIQUE_NAME, botChatConfig)
                    putExtras(openBotViewModel.getExtras())
                })
            }
        }
        botViewContainerView?.getBotView()?.setImageLongClickListener {
            if (openBotViewModel.welcomeDataLiveData.value?.isNotEmpty() == true) {
                botViewContainerView.getWelcomeData()?.visibility = View.VISIBLE
                scrollWelcomeDataToEnd()
                true
            } else {
                false
            }
        }
        botViewContainerView?.getBotView()?.setErrorRetryClickListener {
            openBotViewModel.fetchData()
        }
    }

    private fun setObservers(activity: FragmentActivity) {
        openBotViewModel.isLoadingInitialApi.observe(activity) { value ->
            botViewContainerView?.getBotView()?.setLoading(value)
        }
        openBotViewModel.imageBitmapLiveData.observe(activity) { value ->
            value?.let { bitmap ->
                botViewContainerView?.getBotView()?.setImage(bitmap)
            }
        }
        openBotViewModel.errorStateLiveData.observe(activity) {
            botViewContainerView?.getBotView()?.setErrorView(it != null)
        }
        openBotViewModel.welcomeDataLiveData.observe(activity) {
            if (it?.isNotEmpty() == true) {
                val themeColor = ViewUtils.parseColorSafely(
                    openBotViewModel.botChatWindowConfig?.botThemColor,
                    ViewConstants.VIEW_DEFAULT_COLOR_STRING_DARK
                )
                botViewContainerView?.getWelcomeData()
                    ?.setData(it, themeColor, object : WelcomeVideoView.OnPlayClickListener {
                        override fun onPlayClicked(url: String?) {
                            playerViewModel.startPlaying(url)
                        }

                        override fun onFullScreenClicked() {
                            openFullScreenPlayer(activity, false)
                        }

                        override fun onVolumeToggleClicked() {
                            playerViewModel.toggleVolume()
                        }

                        override fun onPlaybackSpeedChanged(speed: Float) {
                            playerViewModel.setPlaybackSpeed(speed)
                        }

                        override fun onDownloadMenuSelected() {
                            (activity as? PermissionCompatActivity)?.requestReadExternalStoragePermission(
                                REQUEST_READ_EXTERNAL_STORAGE_PERMISSION,
                                this@BotViewHelper
                            )
                        }

                        override fun onPictureInPictureSelected() {
                            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N
                                && activity.packageManager.hasSystemFeature(PackageManager.FEATURE_PICTURE_IN_PICTURE)
                            ) {
                                openFullScreenPlayer(activity, true)
                            } else {
                                activity.showToast(activity.getString(R.string.service_unavailable))
                            }
                        }
                    })
                botViewContainerView?.getWelcomeData()?.visibility = View.VISIBLE
                botViewContainerView?.getWelcomeData()?.setPlayer(playerViewModel.player)
                botViewContainerView?.getWelcomeData()
                    ?.setVolumeMuted(playerViewModel.volumeMutedLiveData.value)
                botViewContainerView?.getWelcomeData()
                    ?.setPlaybackSpeed(playerViewModel.playbackSpeedLiveData.value ?: 1f)
                botViewContainerView?.getWelcomeData()
                    ?.setPlayerPIPMode(playerViewModel.pictureInPictureModeLiveData.value)
                scrollWelcomeDataToEnd()
            } else {
                botViewContainerView?.getWelcomeData()?.visibility = View.GONE
            }
        }
        openBotViewModel.welcomeVideoUrlLiveData.observe(activity) { url ->
            playerViewModel.startPlaying(url)
        }
        playerViewModel.playerLiveData.observe(activity) {
            botViewContainerView?.getWelcomeData()?.setPlayer(it)
            scrollWelcomeDataToEnd()
        }
        playerViewModel.volumeMutedLiveData.observe(activity) {
            botViewContainerView?.getWelcomeData()?.setVolumeMuted(it)
        }
        playerViewModel.playbackSpeedLiveData.observe(activity) {
            botViewContainerView?.getWelcomeData()
                ?.setPlaybackSpeed(it ?: 1f)
        }
        playerViewModel.pictureInPictureModeLiveData.observe(activity) {
            botViewContainerView?.getWelcomeData()?.setPlayerPIPMode(it)
        }
        openBotViewModel.botImageDimensionsLiveData.observe(activity) { botDimension ->
            botViewContainerView?.getBotView()?.setBotDimensions(botDimension)
        }
    }

    private fun openFullScreenPlayer(activity: FragmentActivity, playInPIP: Boolean) {
        playerViewModel.getUrl()?.let { url ->
            val playerSize = botViewContainerView?.getWelcomeData()?.getPlayerSize()
            ExoPlayerActivity.getRequestIntent(
                activity,
                playerViewModel.getCurrentPosition(),
                playerViewModel.getPlayWhenReady(),
                playerViewModel.getPlaybackSpeed(),
                url,
                playInPIP,
                if (playerViewModel.videoWidth > 0) playerViewModel.videoWidth else playerSize?.width
                    ?: 0,
                if (playerViewModel.videoHeight > 0) playerViewModel.videoHeight else playerSize?.height
                    ?: 0,
                playerViewModel.player?.isPlaying,
                true
            ).also { intent ->
                playerViewModel.stopPlaying()
                exoPlayerResult.launch(intent)
            }
        }
    }

    private fun scrollWelcomeDataToEnd() {
        botViewContainerView?.getWelcomeDataContainer()?.postDelayed({
            botViewContainerView.getWelcomeDataContainer()?.fullScroll(View.FOCUS_DOWN)
        }, 600)
    }

    fun onConfigurationChanged(newConfig: Configuration) {
        scrollWelcomeDataToEnd()
    }

    override fun onPermissionResponse(
        requestCode: Int,
        readExternalStoragePermission: PermissionCompatActivity.ReadExternalStoragePermission
    ) {
        if (readExternalStoragePermission == PermissionCompatActivity.ReadExternalStoragePermission.GRANTED) {
            if (requestCode == REQUEST_READ_EXTERNAL_STORAGE_PERMISSION) {
                playerViewModel.downloadVideo()
            }
        }
    }

    override fun onDestroy(owner: LifecycleOwner) {
        super.onDestroy(owner)
        (owner as? FragmentActivity)?.let { activity ->
            try {
                LocalBroadcastManager.getInstance(activity)
                    .unregisterReceiver(videoPlayerPIPBroadcastReceiver)
                LocalBroadcastManager.getInstance(activity)
                    .unregisterReceiver(videoPlayerPositionBroadcastReceiver)
            } catch (e: Exception) {
                LogUtils.logException(e)
            }
        }
    }

    companion object {
        const val RESULT_SESSION_UPDATE_REQUESTED = 999
        const val RESULT_BOT_CONFIG_UPDATED = 1000
        const val REQUEST_READ_EXTERNAL_STORAGE_PERMISSION = 1001
    }
}
